# -*- coding: utf-8 -*-

from m3.actions.utils import extract_int
from m3_ext.ui import all_components as ext

from kinder.core.declaration.models import Declaration, DeclarationUnit
from kinder.core.declaration.enum import DeclarationSourceEnum
from kinder.controllers import obs
from concentrator.models import (ChangeStatus,
                                                ChangeDeclaration)
from concentrator.models import PrivilegeComment
from concentrator.changes.forms import (
    DeclarationEditWindowExtension,
    DeclarationListWindowExtension)


@obs.subscribe
class AddChangesTab(object):
    """
    Добавляем вкладку "Изменения с ЕПГУ"
    """

    listen = [
        'kinder.core.queue_module.declaration/'
        'QueueDeclarationPack/QueueDeclarationEditAction'
    ]

    def configure_edit_window(self, params):
        win = params['win']
        DeclarationEditWindowExtension.build_extension(win)
        return params


@obs.subscribe
class AddCommentPrivileges(object):
    """ Добавляем поле Комментарий "Изменения с ЕПГУ" """

    listen = [
        'kinder.core.declaration.declaration_privilege/'
        'DeclarationPrivilegePack/ObjectEditWindowAction',
        'kinder.core.declaration.declaration_privilege/'
        'DeclarationPrivilegePack/ObjectAddWindowAction'
    ]

    def after(self, request, context, result):
        if not context.decl_privilege_id:
            return

        win = result.data

        try:
            win.comment_privileges = ext.ExtStringField(
                label=u'Комментарий из концентратора', anchor='100%',
                name='comment_privileges', read_only=True,
                value=PrivilegeComment.objects.get(
                    declaration_privilege=context.decl_privilege_id
                ).concentrator_comment)
            win.form.items.append(win.comment_privileges)
        except PrivilegeComment.DoesNotExist:
            pass


@obs.subscribe
class MakeNeedConfirmationExtension(object):
    """
    Добавляем checkbox "Отобразить только заявки на подтверждение"
    """

    listen = [
        'kinder.core.queue_module.declaration/'
        'QueueDeclarationPack/QueueDeclaratonListWindowAction'
    ]

    def configure_list_window(self, params):
        win = params['win']
        DeclarationListWindowExtension.build_extension(win)
        return params


@obs.subscribe
class MakeObjectRowsActionExtension(object):
    """
    Перехватываем rows action. Добавляем фильтр заявок на подтверждение.
    Добавляем метку для подсветки записей с необработанными изменениями.
    """

    listen = [
        'kinder.core.queue_module.declaration/'
        'QueueDeclarationPack/ObjectRowsAction'
    ]

    def get_rows(self, res):
        for row in res:
            row['changes_await'] = ChangeDeclaration.objects.filter(
                declaration_id=row['id'], state=ChangeStatus.WAIT).exists()
        return res

    def apply_filter(self, query):
        wait_only = extract_int(self.request, 'need_confirmation_only')

        if wait_only:
            query = query.filter(
                changedeclaration__state=ChangeStatus.WAIT).distinct()
        return query
