# coding: utf-8
from __future__ import absolute_import, unicode_literals

from concentrator.smev3.base.utils import FEMALE, MALE
from kinder.core.children.models import DelegateTypeEnumerate, GenderEnumerate
from kinder.core.dict.models import DULDelegateType, WorkType


class Rule(type):

    def __new__(mcs, *args, **kwargs):
        rule_cls = super(Rule, mcs).__new__(mcs, *args, **kwargs)
        rule_cls.OUT = getattr(
            rule_cls, 'OUT', {val: key for key, val in rule_cls.IN.items()})
        return rule_cls


class DictRule(object):
    __metaclass__ = Rule

    IN = {}

    @classmethod
    def system_value(cls, concentrator_value):
        return cls.IN.get(concentrator_value)

    @classmethod
    def concentrator_value(cls, system_value):
        return cls.OUT.get(system_value)


class PrivilegeRule(DictRule):
    """Правило соотвествия льгот."""

    IN = {
        '5': 30,  # Дети родителей и врачей
        '19': 7,  # Дети военнослужащих
    }


class ChildGenderRule(DictRule):
    """Правило соответствия пола ребенка."""

    IN = {
        MALE: GenderEnumerate.MALE,
        FEMALE: GenderEnumerate.FEMALE,
    }


class DulDelegateTypeRule(DictRule):
    """Правило соответствия типа ДУЛ родителя."""

    IN = {
        '1': DULDelegateType.RF_PASSPORT,
    }


class WorktypeRule(DictRule):
    """Правило соответствия режима работы."""

    IN = {
        '2': WorkType.ALLDAY,
    }


class DelegateTypeRule(DictRule):
    """Правило соответствия типа представителя."""

    IN = {
        '1': DelegateTypeEnumerate.MOTHER,
        '2': DelegateTypeEnumerate.FATHER,
        '3': DelegateTypeEnumerate.LEX,
    }
